function featureVector = sequentialSearch(X, Ycategories, numWantedFeatures,...
    coding, useMLToolboxSeq, ordinalOrder)
%SEQUENTIALSEARCH performs sequential search feature selection 
%   on data in table X, with
%   each row of X being a sample and each column being a feature, and
%   categorical vector Ycategories containing the targets.
%   numFeatures: maximun number of features to be selected.
%   useMLToolboxECOC: boolean indicating what training function to use
%   useMLToolboxSeq: boolean indication what search function to use

global g_coding
global g_ordinalOrder

g_coding = coding;
g_ordinalOrder = ordinalOrder;
Xarray = table2array(X);
Ycellstr = cellstr(Ycategories);

if useMLToolboxSeq
    %-------------------------------
    % the sequential search implementation from the toolbox is used
    

    % get the recommended input features
    [~,history] = sequentialfs(@critfun,Xarray,...
        Ycellstr, 'cv', 'none', 'options', statset('display','iter'));
    In = history.In;
    
    % convert the outputs from sequentialfs
    In = [In; zeros(1,size(In,2))];
    featureOrder = In - circshift(In,1,1);
    featureOrder = featureOrder(1:end-1,:);
    [featureOrder,~] = find(featureOrder');
 
    % featureOrder now contains the IDs of the selected features
    % in order of their selection
    featureVector = featureOrder;
  
    %-------------------------------
else
    %-------------------------------
    % the open implenemtation is used
    
    numFeatures = size(X,2);
    featureVector = [];
    bestPerformance = 0;
    
    in = zeros(1,numFeatures);
    
    while length(featureVector) < numWantedFeatures
        disp(['#Features included: ' int2str(length(featureVector))]);
        bestLocalPerformance = 0;
        bestLocalFeatures = [];
        bestLocalFeatureIndex = 0;
        for i=1:numFeatures
            if (in(i) == 0)
                disp(['Trying to add feature ' int2str(i)]);
                activeFeatures = [featureVector, i];
                performance = 1 - critfun(Xarray(:,activeFeatures),Ycellstr);
                if performance > bestLocalPerformance
                    bestLocalPerformance = performance;
                    bestLocalFeatures = activeFeatures;
                    bestLocalFeatureIndex = i;
                end
            end
        end
        disp(['Best new feature was ' int2str(bestLocalFeatureIndex)]);
        if (bestLocalPerformance > bestPerformance)
            disp('and it was added to the feature list.');
            bestPerformance = bestLocalPerformance;
            featureVector = bestLocalFeatures;
            in(bestLocalFeatureIndex) = 1;
        else
            disp('but the algorithm was not improved by it.');
            break;
        end
    end
    disp('Final result:')
    disp(featureVector);
    
    %-------------------------------
end

end

% local criterion function
function criterion = critfun(X,Y)
    global g_coding
    global g_ordinalOrder
    Mdl = executeECOC_SVMTraining(X,Y, g_coding, g_ordinalOrder);
    criterion = loss(Mdl, X, Y); % in-sample error
end

